@@ -1,7 +1,7 @@
-use preserves::{de, value::{self, Reader, PackedReader}};
+use preserves::{de, value::{self, Reader, IOBinarySource, BinarySource}};
 use serde::{Serialize, Deserialize};
 use std::fs::File;
-use std::io::Error;
+use std::io;
 
 #[derive(Debug, Serialize, Deserialize, PartialEq)]
 enum Fruit {
@@ -32,8 +32,8 @@ enum Variety {
     Anjou,
 }
 
-fn try_file(kind: &str, path: &str) -> Result<(), Error> {
-    let fruits_value = PackedReader::decode_read(&mut File::open(path)?).demand_next(true)?;
+fn try_file(kind: &str, path: &str) -> io::Result<()> {
+    let fruits_value = IOBinarySource::new(&mut File::open(path)?).packed_iovalues().demand_next(true)?;
     println!("{:?}", fruits_value);
 
     let fruits1: Vec<Fruit> = value::de::from_value(&fruits_value)?;
@@ -46,7 +46,7 @@ fn try_file(kind: &str, path: &str) -> Result<(), Error> {
     Ok(())
 }
 
-fn main() -> Result<(), Error> {
+fn main() -> io::Result<()> {
     try_file("KNOWN", "examples/known-data.bin")?;
     try_file("UNKNOWN", "examples/unknown-data.bin")?;
     Ok(())
