@@ -1,15 +1,19 @@
 use serde::Deserialize;
 use serde::de::{Visitor, SeqAccess, MapAccess, EnumAccess, VariantAccess, DeserializeSeed};
+
 use std::borrow::Cow;
+use std::io;
 use std::marker::PhantomData;
-use super::value::PackedReader;
+
+use super::value::boundary as B;
+use super::value::{IOValue, IOValueDomainCodec, PackedReader, TextReader, ViaCodec};
 use super::value::reader::{Reader, IOBinarySource, BytesBinarySource};
 
 pub use super::error::Error;
 
 pub type Result<T> = std::result::Result<T, Error>;
 
-pub struct Deserializer<'de, 'r, R: Reader<'de>> {
+pub struct Deserializer<'de, 'r, R: Reader<'de, IOValue, IOValue>> {
     pub read: &'r mut R,
     phantom: PhantomData<&'de ()>,
 }
@@ -19,18 +23,23 @@ pub fn from_bytes<'de, T>(bytes: &'de [u8]) ->
 where
     T: Deserialize<'de>
 {
-    from_reader(&mut PackedReader::new(BytesBinarySource::new(bytes)))
+    from_reader(&mut PackedReader::new(&mut BytesBinarySource::new(bytes), IOValueDomainCodec))
 }
 
-pub fn from_read<'de, 'r, IOR: std::io::Read, T>(read: &'r mut IOR) ->
+pub fn from_text<'de, T>(text: &'de str) -> Result<T> where T: Deserialize<'de> {
+    from_reader(&mut TextReader::new(&mut BytesBinarySource::new(text.as_bytes()),
+                                     ViaCodec::new(IOValueDomainCodec)))
+}
+
+pub fn from_read<'de, 'r, IOR: io::Read + io::Seek, T>(read: &'r mut IOR) ->
     Result<T>
 where
     T: Deserialize<'de>
 {
-    from_reader(&mut PackedReader::new(IOBinarySource::new(read)))
+    from_reader(&mut PackedReader::new(&mut IOBinarySource::new(read), IOValueDomainCodec))
 }
 
-pub fn from_reader<'r, 'de, R: Reader<'de>, T>(read: &'r mut R) ->
+pub fn from_reader<'r, 'de, R: Reader<'de, IOValue, IOValue>, T>(read: &'r mut R) ->
     Result<T>
 where
     T: Deserialize<'de>
@@ -40,13 +49,13 @@ where
     Ok(t)
 }
 
-impl<'r, 'de, R: Reader<'de>> Deserializer<'de, 'r, R> {
+impl<'r, 'de, R: Reader<'de, IOValue, IOValue>> Deserializer<'de, 'r, R> {
     pub fn from_reader(read: &'r mut R) -> Self {
         Deserializer { read, phantom: PhantomData }
     }
 }
 
-impl<'r, 'de, 'a, R: Reader<'de>> serde::de::Deserializer<'de> for &'a mut Deserializer<'de, 'r, R>
+impl<'r, 'de, 'a, R: Reader<'de, IOValue, IOValue>> serde::de::Deserializer<'de> for &'a mut Deserializer<'de, 'r, R>
 {
     type Error = Error;
 
@@ -144,31 +153,30 @@ impl<'r, 'de, 'a, R: Reader<'de>> serde::de::Deserializer<'de> for &'a mut Deser
 
     fn deserialize_option<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
     {
-        let is_some = self.read.open_option()?;
-        let result = if is_some {
-            self.read.ensure_more_expected()?;
-            visitor.visit_some(&mut *self)?
+        if let Some(mut b) = self.read.open_option()? {
+            self.read.ensure_more_expected(&mut b, &B::Item::RecordField)?;
+            let result = visitor.visit_some(&mut *self)?;
+            self.read.ensure_complete(b, &B::Item::RecordField)?;
+            Ok(result)
         } else {
-            visitor.visit_none::<Error>()?
-        };
-        self.read.ensure_complete()?;
-        Ok(result)
+            Ok(visitor.visit_none::<Error>()?)
+        }
     }
 
     fn deserialize_unit<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de>
     {
-        self.read.open_simple_record("tuple", Some(0))?;
+        let b = self.read.open_simple_record("tuple", Some(0))?;
         let result = visitor.visit_unit::<Error>()?;
-        self.read.ensure_complete()?;
+        self.read.ensure_complete(b, &B::Item::RecordField)?;
         Ok(result)
     }
 
     fn deserialize_unit_struct<V>(self, name: &'static str, visitor: V)
                                   -> Result<V::Value> where V: Visitor<'de>
     {
-        self.read.open_simple_record(name, Some(0))?;
+        let b = self.read.open_simple_record(name, Some(0))?;
         let result = visitor.visit_unit::<Error>()?;
-        self.read.ensure_complete()?;
+        self.read.ensure_complete(b, &B::Item::RecordField)?;
         Ok(result)
     }
 
@@ -176,14 +184,14 @@ impl<'r, 'de, 'a, R: Reader<'de>> serde::de::Deserializer<'de> for &'a mut Deser
                                      -> Result<V::Value> where V: Visitor<'de>
     {
         match super::value::magic::transmit_input_value(
-            name, || Ok(self.read.demand_next(false)?))?
+            name, || Ok(self.read.demand_next(true)?))?
         {
             Some(v) => visitor.visit_u64(v),
             None => {
-                self.read.open_simple_record(name, Some(1))?;
-                self.read.ensure_more_expected()?;
+                let mut b = self.read.open_simple_record(name, Some(1))?;
+                self.read.ensure_more_expected(&mut b, &B::Item::RecordField)?;
                 let result = visitor.visit_newtype_struct(&mut *self)?;
-                self.read.ensure_complete()?;
+                self.read.ensure_complete(b, &B::Item::RecordField)?;
                 Ok(result)
             }
         }
@@ -192,14 +200,14 @@ impl<'r, 'de, 'a, R: Reader<'de>> serde::de::Deserializer<'de> for &'a mut Deser
     fn deserialize_seq<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de> {
         // Hack around serde's model: Deserialize *sets* as sequences,
         // too, and reconstruct them as Rust Sets on the visitor side.
-        self.read.open_sequence_or_set()?;
-        visitor.visit_seq(Seq::new(self))
+        let i = self.read.open_sequence_or_set()?;
+        visitor.visit_seq(Seq::new(self, B::Type::default(), i))
     }
 
     fn deserialize_tuple<V>(self, len: usize, visitor: V) -> Result<V::Value> where V: Visitor<'de>
     {
-        self.read.open_simple_record("tuple", Some(len))?;
-        let mut seq = Seq::new(self);
+        let b = self.read.open_simple_record("tuple", Some(len))?;
+        let mut seq = Seq::new(self, b, B::Item::RecordField);
         let result = visitor.visit_seq(&mut seq)?;
         seq.skip_remainder()?;
         Ok(result)
@@ -208,8 +216,8 @@ impl<'r, 'de, 'a, R: Reader<'de>> serde::de::Deserializer<'de> for &'a mut Deser
     fn deserialize_tuple_struct<V>(self, name: &'static str, len: usize, visitor: V)
                                    -> Result<V::Value> where V: Visitor<'de>
     {
-        self.read.open_simple_record(name, Some(len))?;
-        let mut seq = Seq::new(self);
+        let b = self.read.open_simple_record(name, Some(len))?;
+        let mut seq = Seq::new(self, b, B::Item::RecordField);
         let result = visitor.visit_seq(&mut seq)?;
         seq.skip_remainder()?;
         Ok(result)
@@ -217,7 +225,7 @@ impl<'r, 'de, 'a, R: Reader<'de>> serde::de::Deserializer<'de> for &'a mut Deser
 
     fn deserialize_map<V>(self, visitor: V) -> Result<V::Value> where V: Visitor<'de> {
         self.read.open_dictionary()?;
-        let mut seq = Seq::new(self);
+        let mut seq = Seq::new(self, B::Type::default(), B::Item::DictionaryKey);
         let result = visitor.visit_map(&mut seq)?;
         Ok(result)
     }
@@ -228,8 +236,8 @@ impl<'r, 'de, 'a, R: Reader<'de>> serde::de::Deserializer<'de> for &'a mut Deser
                              visitor: V)
                              -> Result<V::Value> where V: Visitor<'de>
     {
-        self.read.open_simple_record(name, Some(fields.len()))?;
-        let mut seq = Seq::new(self);
+        let b = self.read.open_simple_record(name, Some(fields.len()))?;
+        let mut seq = Seq::new(self, b, B::Item::RecordField);
         let result = visitor.visit_seq(&mut seq)?;
         seq.skip_remainder()?;
         Ok(result)
@@ -258,51 +266,58 @@ impl<'r, 'de, 'a, R: Reader<'de>> serde::de::Deserializer<'de> for &'a mut Deser
     }
 }
 
-pub struct Seq<'de, 'r, 'a, R: Reader<'de>> {
+pub struct Seq<'de, 'r, 'a, R: Reader<'de, IOValue, IOValue>> {
+    b: B::Type,
+    i: B::Item,
     de: &'a mut Deserializer<'de, 'r, R>,
 }
 
-impl<'de, 'r, 'a, R: Reader<'de>> Seq<'de, 'r, 'a, R> {
-    fn new(de: &'a mut Deserializer<'de, 'r, R>) -> Self {
-        Seq { de }
+impl<'de, 'r, 'a, R: Reader<'de, IOValue, IOValue>> Seq<'de, 'r, 'a, R> {
+    fn new(de: &'a mut Deserializer<'de, 'r, R>, b: B::Type, i: B::Item) -> Self {
+        Seq { b, i, de }
     }
 
     fn skip_remainder(&mut self) -> Result<()> {
-        self.de.read.skip_remainder()
+        while !self.de.read.close_compound(&mut self.b, &self.i)? {
+            self.de.read.skip_value()?;
+        }
+        Ok(())
     }
 
     fn next_item<T>(&mut self, seed: T) ->
         Result<Option<T::Value>> where T: DeserializeSeed<'de>
     {
-        match self.de.read.close_compound()? {
+        match self.de.read.close_compound(&mut self.b, &self.i)? {
             true => Ok(None),
             false => Ok(Some(seed.deserialize(&mut *self.de)?)),
         }
     }
 }
 
-impl<'de, 'r, 'a, R: Reader<'de>> SeqAccess<'de> for Seq<'de, 'r, 'a, R> {
+impl<'de, 'r, 'a, R: Reader<'de, IOValue, IOValue>> SeqAccess<'de> for Seq<'de, 'r, 'a, R> {
     type Error = Error;
 
     fn next_element_seed<T>(&mut self, seed: T) ->
         Result<Option<T::Value>> where T: DeserializeSeed<'de>
     {
-        self.next_item(seed)
+        Ok(self.next_item(seed)?)
     }
 }
 
-impl<'de, 'r, 'a, R: Reader<'de>> MapAccess<'de> for Seq<'de, 'r, 'a, R> {
+impl<'de, 'r, 'a, R: Reader<'de, IOValue, IOValue>> MapAccess<'de> for Seq<'de, 'r, 'a, R> {
     type Error = Error;
 
     fn next_key_seed<K>(&mut self, seed: K) ->
         Result<Option<K::Value>> where K: DeserializeSeed<'de>
     {
+        self.i = B::Item::DictionaryKey;
         self.next_item(seed)
     }
 
     fn next_value_seed<V>(&mut self, seed: V) ->
         Result<V::Value> where V: DeserializeSeed<'de>
     {
+        self.i = B::Item::DictionaryValue;
         match self.next_item(seed)? {
             Some(item) => Ok(item),
             None => Err(Error::MissingItem),
@@ -310,20 +325,20 @@ impl<'de, 'r, 'a, R: Reader<'de>> MapAccess<'de> for Seq<'de, 'r, 'a, R> {
     }
 }
 
-impl<'de, 'r, 'a, R: Reader<'de>> EnumAccess<'de> for &'a mut Deserializer<'de, 'r, R> {
+impl<'de, 'r, 'a, R: Reader<'de, IOValue, IOValue>> EnumAccess<'de> for &'a mut Deserializer<'de, 'r, R> {
     type Error = Error;
     type Variant = Seq<'de, 'r, 'a, R>;
 
     fn variant_seed<V>(self, seed: V)
                        -> Result<(V::Value, Self::Variant)> where V: DeserializeSeed<'de>
     {
-        self.read.open_record(None)?;
+        let b = self.read.open_record(None)?;
         let variant = seed.deserialize(&mut *self)?;
-        Ok((variant, Seq::new(self)))
+        Ok((variant, Seq::new(self, b, B::Item::RecordField)))
     }
 }
 
-impl<'de, 'r, 'a, R: Reader<'de>> VariantAccess<'de> for Seq<'de, 'r, 'a, R> {
+impl<'de, 'r, 'a, R: Reader<'de, IOValue, IOValue>> VariantAccess<'de> for Seq<'de, 'r, 'a, R> {
     type Error = Error;
 
     fn unit_variant(mut self) -> Result<()> {
