@@ -39,6 +39,10 @@ pub trait NestedValue<D: Embeddable>: Sized + Debug + Clone + Eq + Hash + Ord {
         Value::symbol(n).wrap()
     }
 
+    fn bytestring<'a, V: Into<Cow<'a, [u8]>>>(v: V) -> Self {
+        Value::bytestring(v).wrap()
+    }
+
     fn wrap(anns: Annotations<Self, D>, v: Value<Self, D>) -> Self;
 
     fn annotations(&self) -> &Annotations<Self, D>;
@@ -655,6 +659,10 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
         self.as_string().ok_or_else(|| self.expected(ExpectedKind::String))
     }
 
+    pub fn bytestring<'a, V: Into<Cow<'a, [u8]>>>(v: V) -> Self {
+        Value::ByteString(v.into().into_owned())
+    }
+
     pub fn is_bytestring(&self) -> bool {
         self.as_bytestring().is_some()
     }
@@ -746,7 +754,7 @@ impl<N: NestedValue<D>, D: Embeddable> Value<N, D> {
     }
 
     pub fn simple_record(label: &str, expected_arity: usize) -> Record<N> {
-        Self::record(Value::symbol(label).wrap(), expected_arity)
+        Self::record(N::symbol(label), expected_arity)
     }
 
     pub fn simple_record0(label: &str) -> Value<N, D> {
