@@ -373,7 +373,7 @@ impl<'de, 'src, D: Embeddable, Dec: DomainParse<D>, S: BinarySource<'de>> TextRe
             match c {
                 b'(' | b')' | b'{' | b'}' | b'[' | b']' | b'<' | b'>' |
                 b'"' | b';' | b',' | b'@' | b'#' | b':' | b'|' | b' ' =>
-                    return Ok(Value::symbol(&decode_utf8(bs)?).wrap()),
+                    return Ok(N::symbol(&decode_utf8(bs)?)),
                 c => {
                     self.skip()?;
                     bs.push(c)
@@ -409,7 +409,7 @@ impl<'de, 'src, D: Embeddable, N: NestedValue<D>, Dec: DomainParse<D>, S: Binary
             }
             b'|' => {
                 self.skip()?;
-                Value::symbol(&self.read_string(b'|')?).wrap()
+                N::symbol(&self.read_string(b'|')?)
             }
             b';' | b'@' => {
                 if read_annotations {
